#!/usr/bin/python
# -*- coding: utf-8 -*-

# bibleverse.py

import sys
import os
import random
import codecs
from PyQt4 import QtGui
from PyQt4 import QtCore

class BibleVerse(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.setWindowTitle('center')
        self.resize(350, 85)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        self.setWindowFlags(QtCore.Qt.SplashScreen)

        if hasattr(sys, 'frozen'):
            this_file_path = sys.executable
        else:
            this_file_path = __file__

        self._basePath = os.path.dirname(this_file_path) + os.path.sep

        # start timer
        self.timer = QtCore.QBasicTimer()

        # first message
        self.text = ''
        # file with verses
        self.versesFile = self._basePath + 'verses'
        # file to store settings
        self.dataFile = self._basePath + 'data'
        # flag stay on top
        self.alwaysOnTop = False
        # drag position
        self.dragPosition = None;
        # message font
        self.font = QtGui.QFont('Sans', 18)
        # font color
        self._color = QtGui.QColor(0, 0, 0)
        # timer interval
        self.timeInterval = 15000
        # context menu
        self.contextMenu = None

        # flags that set timer
        self.timer5 = False
        self.timer15 = True
        self.timer30 = False
        self.timer60 = False

        # make sure the verses file exists
        if os.access(self.versesFile, os.R_OK):
            file = codecs.open(self.versesFile, 'r', 'utf-8')
            self.content = file.readlines()
            file.close()
        else:
            raise BibleVerseError('verses file not found ' + self.versesFile)

        # make sure verses file is not empty
        if len(self.content) == 0:
            raise BibleVerseError('verses file is empty')
        # выводим первый стих
        self.getBibleVerse()
        # get saved data to position, set timer etc
        self.getData()
        
        self.timer.start(self.timeInterval, self)

    def getBibleVerse(self):
        wordnum = random.randint(0, len(self.content)-1)
        self.text = self.content[wordnum]

    def saveData(self):
        file = open(self.dataFile, 'w+')
        file.truncate(0)
        file.write(str(self.x()) + '|' + str(self.y()))
        file.write("\n")
        file.write(str(self.timeInterval/1000))
        file.write("\n")
        file.write(str(int(self.alwaysOnTop)))
        file.write("\n")
        file.write(QtGui.QFont.toString(self.font))
        file.write("\n")
        r, g, b, a = QtGui.QColor.getRgb(self._color)
        file.write(str(r) + "|" + str(g) + "|" + str(b) + "|" + str(a))
        file.close()

    def getData(self):
        file = open(self.dataFile, 'r')
        lines = file.readlines()
        if len(lines) == 5:
            # if coordinates are stored
            if lines[0].count("|"):
                # move to the saved position
                coord = lines[0].split("|")
                self.move(int(coord[0]), int(coord[1]))
            else:
                # move to the center of the screen
                screen = QtGui.QDesktopWidget().screenGeometry()
                size =  self.geometry()
                self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

            time = int(lines[1])
            if time in [5, 15, 30, 60]:
                self.doSetTimerAction(time)
            
            if str(lines[2]) == 1:
                self.alwaysOnTop = True
                # отображаем        print event поверх всех окон
                self.setWindowFlags(self.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)

            if len(lines[3]) > 0:
                QtGui.QFont.fromString(self.font, lines[3])

            if lines[4].count("|"):
                r, g, b, a = lines[4].split("|")
                QtGui.QColor.setRgb(self._color, int(r), int(g), int(b), int(a))

        # close the file
        file.close()

    def paintEvent(self, event):
        #self.label.setFont(self.font)
        #self.label.setText(self.text)
        #print QtGui.QFont.toString(self.font)
        paint = QtGui.QPainter()
        paint.begin(self)
        paint.setPen(self._color)
        paint.setFont(self.font)
        paint.drawText(event.rect(), QtCore.Qt.TextWordWrap, self.text)
        paint.end()

    def timerEvent(self, event):
        if event.timerId() == self.timer.timerId():
            self.getBibleVerse()
            self.update()
        else:
            QtGui.QFrame.timerEvent(self, event)

    def resetTimer(self):
        self.timer.stop()
        self.timer.start(self.timeInterval, self)

    def mouseMoveEvent(self, event):
        # move the box with the mouse
        self.move(event.globalPos() - self.dragPosition)
        # save current position
        self.saveData()
        event.accept()

    def mousePressEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            # get new bible verse
            self.getBibleVerse()
            # update text
            self.update()
            # get drag position
            self.dragPosition = event.globalPos()-self.frameGeometry().topLeft();
            # reset timer
            self.resetTimer()
            # accept event
            event.accept()    
        elif event.button() == QtCore.Qt.RightButton:
            # если контекстное меню не создано
            if self.contextMenu == None:
                # bsetFontDialoguild context menu
                self.contextMenu = QtGui.QMenu()

                self.actionAlwaysOnTop = QtGui.QAction("Always on top", self)
                self.actionAlwaysOnTop.setCheckable(True)
                self.actionAlwaysOnTop.setChecked(self.alwaysOnTop)
                self.contextMenu.addAction(self.actionAlwaysOnTop)
                self.connect(self.actionAlwaysOnTop, QtCore.SIGNAL("triggered()"), self.alwaysOnTopEvent)

                # set font action
                self.setFont = QtGui.QAction("Set font", self)
                self.contextMenu.addAction(self.setFont)
                self.connect(self.setFont, QtCore.SIGNAL("triggered()"), self.setFontDialog)

                # set widget background
                self.setFontColor = QtGui.QAction("Set font color", self)
                self.contextMenu.addAction(self.setFontColor)
                self.connect(self.setFontColor, QtCore.SIGNAL("triggered()"), self.setFontColorDialog)

                # set font action
                self.setTimerMenu = QtGui.QMenu("Timer", self)
                self.setTimer5 = QtGui.QAction("5 sec", self)
                self.setTimer5.setCheckable(True)
                self.setTimer5.setChecked(self.timer5)
                self.connect(self.setTimer5, QtCore.SIGNAL("triggered()"), self.setTimer5Event)

                self.setTimer15 = QtGui.QAction("15 sec", self)
                self.setTimer15.setCheckable(True)
                self.setTimer15.setChecked(self.timer15)
                self.connect(self.setTimer15, QtCore.SIGNAL("triggered()"), self.setTimer15Event)

                self.setTimer30 = QtGui.QAction("30 sec", self)
                self.setTimer30.setCheckable(True)
                self.setTimer30.setChecked(self.timer30)
                self.connect(self.setTimer30, QtCore.SIGNAL("triggered()"), self.setTimer30Event)

                self.setTimer60 = QtGui.QAction("1 min", self)
                self.setTimer60.setCheckable(True)
                self.setTimer60.setChecked(self.timer60)
                self.connect(self.setTimer60, QtCore.SIGNAL("triggered()"), self.setTimer60Event)

                self.setTimerMenu.addAction(self.setTimer5)
                self.setTimerMenu.addAction(self.setTimer15)
                self.setTimerMenu.addAction(self.setTimer30)
                self.setTimerMenu.addAction(self.setTimer60)
                self.contextMenu.addMenu(self.setTimerMenu)

                #self.connect(self.setFont, QtCore.SIGNAL("triggered()"), self.setFontDialog)

                self.contextMenu.addAction("Exit", QtGui.qApp, QtCore.SLOT("quit()"))

            self.contextMenu.exec_(QtGui.QCursor.pos())

    def alwaysOnTopEvent(self):
        if self.actionAlwaysOnTop.isChecked():
            self.alwaysOnTop = True
            self.setWindowFlags(self.windowFlags() | QtCore.Qt.WindowStaysOnTopHint)
            self.show()
        else:
            self.alwaysOnTop = False
            self.setWindowFlags(QtCore.Qt.Tool | QtCore.Qt.FramelessWindowHint)
            self.show()
            
        self.saveData()

    def doSetTimerAction(self, number):
        for num in [5, 15, 30, 60]:
            if num == number:
                setattr(self, 'timer' + str(num), True)
            else:
                setattr(self, 'timer' + str(num), False)

        self.timeInterval = number * 1000
        # save interval changes to file
        self.saveData()
        self.resetTimer()

    def setTimer5Event(self):
        self.doSetTimerAction(5)
        self.show()

    def setTimer15Event(self):
        self.doSetTimerAction(15)
        self.show()

    def setTimer30Event(self):
        self.doSetTimerAction(30)
        self.show()

    def setTimer60Event(self):
        self.doSetTimerAction(60)
        self.show()

    def setFontDialog(self):
        #return
        #QtGui.QFontDialog.getFont()
        font, ok = QtGui.QFontDialog.getFont(self.font, self)
        if ok:
            self.font = font
            self.update()
            self.saveData()
                        

    '''
    def showBackground(self):
        col = QtGui.QColorDialog.getColor(self.color())

        if col.isValid():
            #self.setStyleSheet("QWidget { background-color: %s }" % col.name())
            self.setStyleSheet("QWidget { background-color: %s }" % col.name())
            #self.update()
            #self.show()'''


    def setFontColorDialog(self):
        color = QtGui.QColorDialog.getColor(self._color, self)
        if color.isValid():
            self._color = color
            self.update()
            self.saveData()


# bible verse exception
class BibleVerseError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


try:
    if __name__ == '__main__':
        app = QtGui.QApplication(sys.argv)
        qb = BibleVerse()
        qb.show()
        sys.exit(app.exec_())
        app.exec_()
except BibleVerseError as e:
    print 'Error occured, message:', e.value

